#!/usr/bin/python

# Webget index in Python
# azz@gnu.org

# This does not yet deal with <base ...> elements.

# 2000/02/27 Made the rewriting rules rewrite to use "view", and to make
# "view" fall through to "add" if the file isn't found.
# 2000/03/01 Made the Content-Type: header only get printed if the file is
# found; added .adp as an HTML-alike (AOLServer embedded Tcl page).
# 2000/05/28 Made the rewriter produce "/~azz/webget..." URLs rather than
# explicitly pointing to cartman, and strip BASE tags.
# 2000/06/25 Made the <a href...> regex use \s for spaces, since then
# it correctly handles <a href= <newline> "something">
# 2000/07/18 Moved a lot of code out into the wgtools module, and made
# this check that a URL has a valid schema

import sys
import os
import string
import cgi
import re
import urllib
import wgtools

limit = 400

sys.stderr = sys.stdout
form = cgi.FieldStorage()
# form = {}

curbase = ""

def rewrite(x):
    if x.group(1) == "":
        return ""
    ileft = x.start(0)
    if x.group(1)[0] == "#":
        return x.group(0)[:x.start(1)-ileft] + x.group(1) + x.group(0)[x.end(1)-ileft:]
    else:
        w = "/webget/wgindex.cgi?view="
        url = wgtools.makeabsolute(x.group(1),curbase)
        # strip off targets
        loc = string.rfind(url, "#")
        if loc > -1:
            url = url[:loc]
        return x.group(0)[:x.start(1)-ileft] + w + urllib.quote(url,"") + x.group(0)[x.end(1)-ileft:]

if form.has_key("view"):
    url = form["view"].value
    filename = wgtools.gotdir + wgtools.filenameof(url)
    ext = wgtools.extensionof(url)
    
    type = "text/plain"
    if ext != "":
        ext = string.lower(ext)

        if ext in ["html", "htm", "php3", "cgi", "shtml", "phtml", "asp", "pt", "xp", "en", "adp"]:
            type = "text/html"
        elif ext == "ps" or ext == "Z" or ext == "gz":
            type = "application/postscript"
        elif ext == "pdf":
            type = "application/pdf"
        elif ext == "gif":
            type = "image/gif"
        elif ext in ["jpg", "jpeg"]:
            type = "image/jpeg"
        elif ext == "png":
            type = "image/png"
    if re.match("^http://.*/$", url) or form.has_key("forcehtml"):
        type = "text/html"

    try:
        file = open(filename)
        print "Content-type: " + type
        print

        text = file.read()
	
	# Remove LINK tags (cancels stylesheets, which prevent browsing in Netscape)
	reg = re.compile("<[/]?link[^>]*>", re.IGNORECASE | re.DOTALL | re.MULTILINE)
	text = reg.sub("", text)

	# Remove BASE tags (this should really pull out the URL and
	# use it for the rewriting)
	reg = re.compile("<[/]?base[^>]*>", re.IGNORECASE | re.DOTALL | re.MULTILINE)
	text = reg.sub("", text)

	# Rewrite A tags to point to wgindex.cgi again
        reg = re.compile("<a[^>]*href\s*=\s*[\'\"]?([^\"\'>]*)[\'\"]?[^>]*>", re.IGNORECASE | re.DOTALL | re.MULTILINE)
	curbase = re.sub("([^/])/[^/]*$", r"\1", form["view"].value)
        text = reg.sub(rewrite, text)

	print text
        file.close()
        sys.exit(0)
    except IOError:
        # This used to print an error, but now we just assume that it needs
        # adding.
        pass


# Not view mode, must be add or list so we need the HTML headers.
# If it still is view mode, we didn't have the file, so we need to add it.
print """Content-type: text/html

<html><head><title>webget control</title></head>
<body>
<h1>Webget control</h1><hr>"""

if form.has_key("add") or form.has_key("requeue") or form.has_key("view"):
    if form.has_key("requeue"):
        addr = form["requeue"].value
        wgtools.remove(addr)
    elif form.has_key("add"): addr = form["add"].value
    elif form.has_key("view"): addr = form["view"].value

    # kludge to cope with regetting old addresses which failed 'cos the remote
    # webserver didn't strip off the target
    loc = string.rfind(addr, "#")
    if loc > -1:
        addr = addr[:loc]

    if wgtools.queue(addr):
        print """<p>'""" + addr + """' has been added to the queue.</body></html>"""
    else:
        print """<p>'""" + addr + """' does not have a valid URL schema."""

    sys.exit(0)

print "<p>The following pages are in the queue:<p><ul>"
d = wgtools.orderdir(wgtools.queuedir)
for k in range(len(d)):
    realname = wgtools.urlof(d[k][0])
    print "<li>(<a href=\"" + realname + "\">original</a>) " + realname
print "</ul><p>The following pages have been received:<p><ul>"
d = wgtools.orderdir(wgtools.gotdir)

count = 0

for k in range(len(d)):
    realname = wgtools.urlof(d[k][0])
    print "<li>(<a href=\"" + realname + "\">orig</a>)"
    print "(<a href=\"wgindex.cgi?view=" + urllib.quote(realname,"") + "\">local</a>)"
    print "(<a href=\"wgindex.cgi?forcehtml=1&view=" + urllib.quote(realname,"") + "\">lochtml</a>)"
    print "(<a href=\"wgindex.cgi?requeue=" + urllib.quote(realname,"") + "\">reget</a>)"
    print realname

    count = count + 1
    if (not form.has_key("nolimit")) and (count > limit):
        break

print """</ul>"""

if (not form.has_key("nolimit")):
    print """<p><a href="wgindex.cgi?nolimit=1">List all items</a>"""
else:
    print """<p><a href="wgindex.cgi">List only top """ + str(limit) + """ items</a>"""

print """<p><form method=get action="wgindex.cgi">
Enter a URL to add: 
<input type=text size=50 name=add value=\"http://\">
<input type=submit value=\"Add\"></form>
<hr>
<p>Comments? <a href=\"mailto:azz@gnu.org\">azz@gnu.org</a>
</body></html>"""

        







